﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Infragistics.Win;
using Infragistics.Win.UltraWinSchedule.MonthViewMulti;
using System.Windows.Forms;
using Infragistics.Win.UltraWinSchedule.CalendarCombo;
using Infragistics.Win.UltraWinSchedule;
using System.Diagnostics;
using System.Drawing;


namespace WinCalendarComboDisableDate
{
    class MyFilters : IUIElementCreationFilter, IUIElementDrawFilter
    {
        List<DateRange> disableDateRanges = new List<DateRange>();

        public List<DateRange> DisableDateRanges
        {
            get { return disableDateRanges; }
            set { disableDateRanges = value; }
        }

        List<DateTime> disableDates = new List<DateTime>();

        public List<DateTime> DisableDates
        {
            get { return disableDates; }
            set { disableDates = value; }
        }


        #region IUIElementCreationFilter Members

        public void AfterCreateChildElements(UIElement parent)
        {

        }

        public bool BeforeCreateChildElements(UIElement parent)
        {
            DayUIElement day = parent as DayUIElement;
            if (day != null) if (IsDisableDate(day.Date))
                {
                    day.Day.Enabled = false;
                }
            return false;
        }

        private bool IsDisableDate(DateTime date)
        {
            bool retvalue = false;
            List<DateTime> retdate = disableDates.Where(d => d.Date == date.Date).ToList<DateTime>();
            List<DateRange> retrange = disableDateRanges.Where(dr => dr.StartDate.Date <= date.Date && date.Date <= dr.EndDate.Date).ToList<DateRange>();
            if (retdate.Count > 0)
                retvalue = true;
            if (retrange.Count > 0)
                retvalue = true;

            return retvalue;

        }

        #endregion

        #region IUIElementDrawFilter Members

        public bool DrawElement(DrawPhase drawPhase, ref UIElementDrawParams drawParams)
        {
            drawParams.AppearanceData.ForeColor = Color.Gray;
            return false;
        }

        public DrawPhase GetPhasesToFilter(ref UIElementDrawParams drawParams)
        {
            if (drawParams.Element is DayUIElement && !((DayUIElement)drawParams.Element).Day.Enabled)
                return DrawPhase.BeforeDrawForeground;
            return DrawPhase.None;
        }

        #endregion
    }

    public class DateRange
    {
        DateTime startDate;

        public DateTime StartDate
        {
            get { return startDate; }
            set
            {
                if (endDate.Date >= value.Date)
                {
                    ArgumentOutOfRangeException ex = new ArgumentOutOfRangeException("StarDate", "StarDate could not be bigger or equal to EndDate");
                    throw ex;
                }
                else
                    startDate = value;
            }
        }
        DateTime endDate;

        public DateTime EndDate
        {
            get { return endDate; }
            set
            {
                if (startDate.Date >= value.Date)
                {
                    ArgumentOutOfRangeException ex = new ArgumentOutOfRangeException("EndDate", "EndDate could not be smaller or equal to StarDate");
                    throw ex;
                }
                else
                    endDate = value;
            }
        }

        public DateRange(DateTime StratDateRange, DateTime EndDateRange)
        {
            if (StratDateRange.Date >= EndDateRange.Date)
            {
                throw new ArgumentOutOfRangeException("EndDate", "EndDate could not be smaller or equal to StarDate");
            }
            else
            {
                startDate = StratDateRange;
                endDate = EndDateRange;
            }

        }
    }
}
